/*
*   timer: print newlines at intervals.
*   92/09/11 John H. DuBois III   john@armory.com
*   95/04/06 Added all options.
*   96/04/23 Added r option.  Let s suffix be used for seconds.
*            Let 0 times be given.  Let multiple times be given on cmd line.
*   
*   Compile with cc -O -o timer timer.c -lx
*/
#include <string.h>
#include <sys/types.h>
#include <unistd.h>

#define WriteString(S,fd) write(fd,S,strlen(S))
#define ErrExit(Message) WriteString(Message,2),exit(1)

long GetTime(char *S);
void DoNaps(unsigned Count,long Time,char *S,size_t Len);

char HelpMsg[] = \
"timer: Print newlines at intervals, for use in script programs.\n"\
"Usage: timer [-hi] [-s<string>] [-r<num>] [<interval> ...]\n" \
"where <interval> is a time to wait between printing newlines.\n"\
"Intervals are given in milliseconds by default.  An 's' may be appended\n"\
"to an interval to give a time in seconds instead of milliseconds.\n"\
"If one interval is given, a newline is printed each time the\n"\
"interval expires.  This continues until the program is interrupted.\n"\
"If more than one interval is given, a newline is printed after each.\n"\
"After the last interval's newline is printed, the program exits.\n"\
"To give a single one-shot interval, use -r1 (see the -r option below).\n"\
"If no intervals are given, wait times are read from the standard input.\n"
"In this mode, timer waits until an interval and newline have been read,\n"\
"waits for the specied interval, writes a newline, and then reads again.\n" \
"Multiple intervals can be given on a line, separated by newlines.\n" \
"If a bad interval is read or the standard input is closed, timer exits.\n" \
"Note that on most systems, the clock period is at least 10 milliseconds.\n" \
"Interval requests are silently rounded up to the next highest clock period.\n"\
"Options:\n"\
"-h: Print this help.\n"\
"-i: Print an initial newline.  timer normally waits until after the first\n"\
"    interval before printing anything.  In some cases it may be useful for\n"\
"    the first iteration of a loop that reads from timer to execute\n"\
"    immediately; -i can be used in this case.  This is not counted against\n"\
"    the value given with -r.\n"\
"-r<num>: Repeat all intervals <num> times.  If intervals are given on the\n"\
"    command line, timer exits after the last interval is processed, even\n"\
"    if only one interval is given.\n"\
"-s<string>: Change the string printed after each interval (and before the\n"\
"    first interval, if -i is given) from a single newline to whatever\n"\
"    string is given with -s.  A newline is not automatically printed after\n"\
"    the string.\n"\
"-S<string>: Like -s, except that the string is followed by a newline.\n";

char rs[] = "@(#) timer 2.0 96/04/23";

#define MAXLINE 1024

main(int argc, char *argv[])
{
    char Line[MAXLINE];
    char *Number;
    char *S = "\n";
    int Len = 1;
    int DoInitial = 0;
    int Num = 0;

    int Err = 0;
    char c;			/* Option character */
    extern char *optarg;	/* For optargs */
    extern int optind;		/* For optargs */

    while ((c = getopt(argc,argv,"his:S:r:")) != -1)
	switch (c) {
	case 'h':
	    WriteString(HelpMsg,1);
	    exit(0);
	case 'i':
	    DoInitial = 1;
	    break;
	case 'S':
	    /* Since we are using write(), don't need string to be null
	    terminated, so replace null term with newline. */
	    Len = strlen(optarg);
	    optarg[Len++] = '\n';
	    S = optarg;
	    break;
	case 'r':
	    if ((Num = atoi(optarg)) < 1)
		ErrExit("timer: Bad value given with -r.\n");
	    break;
	case 's':
	    S = optarg;
	    Len = strlen(optarg);
	    break;
	case '?':
	    Err = 1;
	}
    if (Err)
	ErrExit("Use -h for help.\n");
    argc -= optind;
    /* If more than one interval given on cmd line and -r not given,
       use each interval once */
    if (argc > 1 && !Num)
	Num = 1;
    if (DoInitial)
	write(1,S,Len);
    if (argc >= 1)
	while (argc--)
	    /* This will never return if Num is 0 */
	    DoNaps(Num,GetTime(argv[optind++]),S,Len);
    else while (ReadLine(Line,MAXLINE,0) != -1) {
	Number = strtok(Line," \t");
	while (Number) {
	    DoNaps(Num,GetTime(Number),S,Len);
	    Number = strtok((char *) 0," \t");
	}
    }
}

/*
 * Sleep Time, then print S.
 * Repeat Count times, or forever if Count is 0.
 */
void DoNaps(unsigned Count,long Time,char *S,size_t Len)
{
    if (Count)
	Count++;
    while (!Count || --Count) {
	nap(Time);
	write(1,S,Len);
    }
}

long GetTime(char *S)
{
    char *p;
    long val;

    val = strtol(S,&p,0);
    if (*p == 's')	/* Do this 1st to catch err if it causes wraparound */
	val *= 1000;
    if (p == S || val < 0 || *p != 's' && *p != '\0') {
	WriteString("timer: Bad time: ",2);
	WriteString(S,2);
	ErrExit("\n");
    }
    return val;
}

/*
*   ReadLine: read a line from a file descriptor one character at at time.
*   Up to Len - 1 characters are read into string S from file descriptor fd.
*   Reading ends when Len - 1 characters are read, the read fails,
*   or a newline is read.
*   The string is null terminated.  The newline, if any, is discarded.
*   
*   Return value:
*   The number of characters put in S is returned.
*   If a read fails and no characters have been put in S, -1 is returned.
*/

int ReadLine(char S[], int Len, int fd)
{
    int i;
    char C = '\0';

    Len--;
    for (i = 0; i < Len && read(fd,&C,1) == 1 && C != '\n'; i++)
	S[i] = C;
    S[i] = '\0';
    return((i || C == '\n') ? i : -1);
}
